﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8
@EndOfDesignText@
' ExpandViews - Class module
' V. 1.0
' Author: LucaMs
Sub Class_Globals
	Private xui As XUI 'ignore
	
	Public Const EXPVW_DIR_LEFT_TO_RIGHT As Int = 1
	Public Const EXPVW_DIR_RIGHT_TO_LEFT As Int = 2
	Public Const EXPVW_DIR_TOP_TO_BOTTOM As Int = 3
	Public Const EXPVW_DIR_BOTTOM_TO_TOP As Int = 4
	
	Public Const EXPVW_ANIM_INVERT As Int = 0
	Public Const EXPVW_ANIM_COLLAPSE As Int = 1
	Public Const EXPVW_ANIM_EXPAND As Int = 2

	Private mmapViews As Map
	
	Private Const ANIM_DURATION As Int = 400
	
	Type tExpVwItem(Vw As B4XView, CollapsedMinSize As Int, Duration As Int, Direction As Int, IsCollapsed As Boolean)
	
	Private noaScaleX, noaScaleY, noaScale As noaObjectAnimator
	Private noaHelper As noaViewHelper
End Sub

'Initializes the object.
Public Sub Initialize
	mmapViews.Initialize
End Sub

' [Vw] a B4XView Or View to animate.
' [Duration] pass -1 to use the default, 400 ms.
' [Direction] pass one of the constants provided, which start with EXPVW_DIR_.
'      pass -1 to use the default, EXPVW_DIR_LEFT_TO_RIGHT.
' [CollapsedMinSize] usually zero but you may want to set a minimum of size to be able to touch
'      and expand the view. It is, obviously, related to the Direction.
Public Sub AddView(Vw As B4XView, CollapsedMinSize As Int, Duration As Int, Direction As Int)
	If mmapViews.Size = 0 Then
		noaScaleX.InitializeFloat(Vw, noaScaleX.PROPERTY_SCALE_X, 0, 1, "noaScaleX")
		noaScaleY.InitializeFloat(Vw, noaScaleY.PROPERTY_SCALE_Y, 0, 1, "noaScaleY")
	End If

	Dim ExpVwItem As tExpVwItem
	ExpVwItem.Initialize
	ExpVwItem.Vw = Vw
	CollapsedMinSize = Max(0, CollapsedMinSize)
	ExpVwItem.CollapsedMinSize = CollapsedMinSize
	If Duration = - 1 Then
		ExpVwItem.Duration = ANIM_DURATION
	Else
		ExpVwItem.Duration = Max(0, Duration)
	End If
	If Direction = - 1 Then
		ExpVwItem.Direction = EXPVW_DIR_LEFT_TO_RIGHT
	Else
		ExpVwItem.Direction = Direction
	End If
	Select ExpVwItem.Direction
		Case EXPVW_DIR_TOP_TO_BOTTOM
			noaHelper.setPivotY(Vw, 0)
		Case EXPVW_DIR_BOTTOM_TO_TOP
			noaHelper.setPivotY(Vw, Vw.Height)
		Case EXPVW_DIR_LEFT_TO_RIGHT
			noaHelper.setPivotX(Vw, 0)
		Case EXPVW_DIR_RIGHT_TO_LEFT
			noaHelper.setPivotX(Vw, Vw.Width)
	End Select
	ExpVwItem.IsCollapsed = False

	mmapViews.Put(Vw, ExpVwItem)
End Sub

'Expands or collapses the view (depending on the current state of the view).
'[Vw] View or B4XView to animate. It must have been previously added using AddView.
'[Duration] pass -1 to use the default (400 ms) or the last on you used.
Public Sub Animate(Vw As B4XView, Duration As Int)
	Dim ExpVwItem As tExpVwItem = mmapViews.Get(Vw)
	ExpVwItem = mmapViews.Get(Vw)
	
	noaScaleX.Target = Vw
	noaScaleY.Target = Vw
	
	If Duration <> - 1 Then
		ExpVwItem.Duration = Max(0, Duration)
	End If
	
	If ExpVwItem.isCollapsed Then
		' to be expanded.
		Select ExpVwItem.Direction
			Case EXPVW_DIR_BOTTOM_TO_TOP, EXPVW_DIR_TOP_TO_BOTTOM
				noaScaleY.FloatValues = Array As Float(0, 1)
				noaScale = noaScaleY
			Case EXPVW_DIR_LEFT_TO_RIGHT, EXPVW_DIR_RIGHT_TO_LEFT
				noaScaleX.FloatValues = Array As Float(0, 1)
				noaScale = noaScaleX
		End Select
	Else
		' to be collapsed.
		Dim MinSize As Float
		Select ExpVwItem.Direction
			Case EXPVW_DIR_BOTTOM_TO_TOP, EXPVW_DIR_TOP_TO_BOTTOM
				MinSize = ExpVwItem.CollapsedMinSize / Vw.Height
				noaScaleY.FloatValues = Array As Float(1, MinSize)
				noaScale = noaScaleY
			Case EXPVW_DIR_LEFT_TO_RIGHT, EXPVW_DIR_RIGHT_TO_LEFT
				MinSize = ExpVwItem.CollapsedMinSize / Vw.Width
				noaScaleX.FloatValues = Array As Float(1, MinSize)
				noaScale = noaScaleX
		End Select
	End If

	noaScale.Start
	
	ExpVwItem.isCollapsed = Not(ExpVwItem.isCollapsed)
End Sub

